/**
 *  -----------------------------------------------------------------------------
 *  
 *  Holo-Edit, spatial sound trajectories editor, part of Holophon
 *  Copyright (C) 2006 GMEM
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 *  
 *  -----------------------------------------------------------------------------
 */
package holoedit.fileio;

import holoedit.Main;
import holoedit.data.HoloRecentFile;
import holoedit.data.HoloSpeaker;
import holoedit.gui.LabelAudio;
import holoedit.util.Ut;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import javax.swing.UIManager;
import javax.swing.plaf.metal.MetalLookAndFeel;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import org.xml.sax.Attributes;
import org.xml.sax.helpers.DefaultHandler;

public class HoloSettings extends DefaultHandler
{
	// BOOLEEN POUR TRAVAILLER AVEC OU SANS SAUVEGARDE DES SETTINGS
	final public static boolean SETTINGS = true;
	private double fileversion = 0;
	private Main mainRef;
	// Nom du fichier de configuration
	public String fileSettings = "HoloSettings.cfg";
	private HoloRecentFile lastOne;
	private int fctCpt;
	public String focus = "room";

	public HoloSettings(Main owner)
	{
		mainRef = owner;
	}

	public void save()
	{
		if (SETTINGS)
			try
			{
				new File(fileSettings).renameTo(new File(fileSettings.concat("~")));
				File f = new File(fileSettings);
				FileWriter fw = new FileWriter(f);
				BufferedWriter out = new BufferedWriter(fw);
				out.write("<?xml version=\"1.0\" encoding=\"iso-8859-1\" standalone=\"yes\"?>\n");
				out.write("<settings version=\"" + Ut.version + "\">\n");
				// OPTIONS SETTINGS
				out.write("\t<options " + "maxRecentFiles=\"" + mainRef.maxRecentFiles + "\"" + " allTrackActive=\"" + mainRef.allTrackActive + "\""
						+ " viewOnlyEditablePoints=\"" + mainRef.viewOnlyEditablePoints + "\"" + " viewSpeakers=\"" + mainRef.viewSpeakers + "\""
						+ " openLastOnLoad=\"" + mainRef.openLastOnLoad + "\"" + " drawPtsNb=\"" + Ut.drawPtsNb + "\""
						+ " coordinates=\"" + mainRef.coordinates + "\"" + " laf=\"" + mainRef.laf + "\""
						+ " shortView=\"" + mainRef.shortViewMode + "\""
						+ " focus=\"" + (mainRef.room.hasFocus() ? "room" : (mainRef.score.hasFocus() ? "score" : (mainRef.timeEditor.hasFocus() ? "time" : (mainRef.room3d.hasFocus() ? "room3d" : "room")))) + "\""
						+ " bonjour=\"" + mainRef.bonjour + "\""
						+ " scrollspeed=\"" + mainRef.scrollSpeed + "\""
						+ " smooth=\"" + Main.SMOOTH + "\""
						+ " sdifExpert=\"" + mainRef.sdifExpert + "\""
						+ " lastScriptFile=\"" + mainRef.lastScriptFile + "\""
						+ "/>\n");
				// OSC SETTINGS
				out.write(mainRef.connection.toString());
				// DIRECTORIES
				out.write("\t<holo dir=\"" + mainRef.gestionPistes.holoDirectory + "\"/>\n");
				out.write("\t<midi dir=\"" + mainRef.gestionPistes.midiDirectory + "\"/>\n");
				out.write("\t<midi7 dir=\"" + mainRef.gestionPistes.midi7Directory + "\"/>\n");
				out.write("\t<txt dir=\"" + mainRef.gestionPistes.textDirectory + "\"/>\n");
				out.write("\t<seq dir=\"" + mainRef.gestionPistes.seqDirectory + "\"/>\n");
				out.write("\t<snd dir=\"" + mainRef.gestionPistes.sndDirectory + "\"/>\n");
				out.write("\t<tk dir=\"" + mainRef.gestionPistes.tkDirectory + "\"/>\n");
				out.write("\t<tj dir=\"" + mainRef.gestionPistes.tjDirectory + "\"/>\n");
				// LABELS AUDIO
				for (LabelAudio lab : mainRef.gestionPistes.ts.labelAudio)
					out.write(lab.toString());
				// RECENT FILES
				// last
				out.write((mainRef.last != null ? mainRef.last.toLastString() : ""));
				int cpt = 1;
				// recent
				for (HoloRecentFile h : mainRef.recentFiles)
				{
					if (cpt++ > mainRef.maxRecentFiles)
						break;
					out.write(h.toString());
				}
				// HPs
				for (HoloSpeaker sp : mainRef.gestionPistes.speakers)
					out.write(sp.toString());
				// WINDOWS SIZE & LOCATION
				out.write(mainRef.gestionPistes.ts.toString());
				out.write(mainRef.transport.toString());
				out.write(mainRef.soundPoolGui.toString());
				out.write(mainRef.score.toString());
				out.write(mainRef.room.toString());
				out.write(mainRef.room3d.toString());
				out.write(mainRef.timeEditor.toString());
				// END
				out.write("</settings>\n");
				out.close();
			}
			catch (IOException ioe)
			{
				ioe.printStackTrace();
			}
			catch (SecurityException se)
			{
				se.printStackTrace();
			}
	}

	public void load()
	{
		if (SETTINGS)
		{
			SAXParserFactory factory = SAXParserFactory.newInstance();
			try
			{
				SAXParser saxParser = factory.newSAXParser();
				File f = new File(fileSettings);
				saxParser.parse(f, this);
			}
			catch (Exception e)
			{
				if (e.getClass().getName().equalsIgnoreCase("java.io.FileNotFoundException"))
				{
					System.out.println("Warning : Can't find file HoloSettings.cfg");
				} else {
					Ut.print(" x Error while loading the file, aborted.");
					e.printStackTrace();
				}
				if(!Ut.MAC)
				{
					try
					{
						switch (mainRef.laf)
						{
						case 0:
							UIManager.setLookAndFeel(new MetalLookAndFeel());
							mainRef.updateUI();
							Ut.barMenu.basicLAFMenuItem.setSelected(true);
							break;
						case 1:
							UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
							mainRef.updateUI();
							Ut.barMenu.systemLAFMenuItem.setSelected(true);
							break;
						case 2:
							UIManager.setLookAndFeel("com.sun.java.swing.plaf.motif.MotifLookAndFeel");
							mainRef.updateUI();
							Ut.barMenu.motifLAFMenuItem.setSelected(true);
						default:
							break;
						}
					}
					catch (Exception e1)
					{
						System.err.println("Look and Feel error :");
						e1.printStackTrace();
					}
				}
			}
		}
	}

	public void startDocument()
	{
		Ut.print("___________________________________________");
		Ut.print(" o Reading file : " + fileSettings + " in progress...");
		mainRef.gestionPistes.speakers.clear();
	}

	public void endDocument()
	{
		mainRef.last = lastOne;
		Ut.barMenu.update();
		Ut.print(" v File was successfully read");
		Ut.print("___________________________________________");
	}

	public void startElement(String uri, String localName, String qName, Attributes attrs)
	{
		// <settings>
		if (qName.equalsIgnoreCase("settings"))
		{
			String v = "v";
			fileversion = new Double(attrs.getValue(attrs.getIndex("version")));
			if (fileversion > Ut.version)
			{
				Ut.print(" x ATTENTION : Ce fichier a t cr avec une version plus rcente de Luminaria, " + "il est possible qu'il ne soit pas compatible avec la prsente version.");
				v = "+";
			}
			else if (fileversion < Ut.version)
			{
				Ut.print(" x ATTENTION : Ce fichier a t cr avec une version antrieure de Holo-Edit, " + "si vous le modifiez, il est possible qu'il ne soit plus compatible avec les versions antrieures du logiciel.");
				v = "-";
			}
			Ut.print(" " + v + " Settings - version " + fileversion);
			Ut.print("...");
		}
		else if (qName.equalsIgnoreCase("options"))
		{
			mainRef.maxRecentFiles = new Integer(attrs.getValue("maxRecentFiles"));
			mainRef.allTrackActive = new Boolean(attrs.getValue("allTrackActive"));
			mainRef.viewOnlyEditablePoints = new Boolean(attrs.getValue("viewOnlyEditablePoints"));
			mainRef.viewSpeakers = new Boolean(attrs.getValue("viewSpeakers"));
			mainRef.openLastOnLoad = new Boolean(attrs.getValue("openLastOnLoad"));
			mainRef.bonjour = mainRef.bonjourInstalled ? new Boolean(attrs.getValue("bonjour")) : false;
			Ut.drawPtsNb = new Integer(attrs.getValue("drawPtsNb"));
			mainRef.coordinates = new Integer(attrs.getValue("coordinates"));
			mainRef.scrollSpeed = new Float(attrs.getValue("scrollspeed"));
			mainRef.sdifExpert = new Boolean(attrs.getValue("sdifExpert"));
			mainRef.lastScriptFile = new File(attrs.getValue("lastScriptFile"));
			
			Main.SMOOTH = new Boolean(attrs.getValue("smooth"));
			if(new File("nosmooth").exists())
				Main.SMOOTH = false;			
			
			if(!Ut.MAC)
			{
				mainRef.laf = new Integer(attrs.getValue("laf"));
				try
				{
					switch (mainRef.laf)
					{
					case 0:
						UIManager.setLookAndFeel(new MetalLookAndFeel());
						mainRef.updateUI();
						Ut.barMenu.basicLAFMenuItem.setSelected(true);
						break;
					case 1:
						UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
						mainRef.updateUI();
						Ut.barMenu.systemLAFMenuItem.setSelected(true);
						break;
					case 2:
						UIManager.setLookAndFeel("com.sun.java.swing.plaf.motif.MotifLookAndFeel");
						mainRef.updateUI();
						Ut.barMenu.motifLAFMenuItem.setSelected(true);
					default:
						break;
					}
				}
				catch (Exception e)
				{
					System.err.println("Look and Feel error :");
					e.printStackTrace();
				}
			}
			mainRef.shortViewMode = new Boolean(attrs.getValue("shortView"));
			mainRef.gestionPistes.ts.labelPlusDelta.setSelected(mainRef.shortViewMode);
			focus = attrs.getValue("focus");
		}
		else if (qName.equalsIgnoreCase("osc"))
		{
			mainRef.connection.setIn(new Integer(attrs.getValue("portIn")));
			mainRef.connection.setOut(new Integer(attrs.getValue("portOut")));
			mainRef.connection.setAddress(attrs.getValue("address"));
			mainRef.connection.setKeyIn(attrs.getValue("keyIn"));
			mainRef.connection.setKeyOut(attrs.getValue("keyOut"));
			if (new Boolean(attrs.getValue("open")))
			{
				mainRef.connection.open();
				mainRef.transport.activer();
			}
		}
		else if (qName.equalsIgnoreCase("holo"))
		{
			mainRef.gestionPistes.holoDirectory = attrs.getValue("dir");
		}
		else if (qName.equalsIgnoreCase("midi"))
		{
			mainRef.gestionPistes.midiDirectory = attrs.getValue("dir");
		}
		else if (qName.equalsIgnoreCase("midi7"))
		{
			mainRef.gestionPistes.midi7Directory = attrs.getValue("dir");
		}
		else if (qName.equalsIgnoreCase("txt"))
		{
			mainRef.gestionPistes.textDirectory = attrs.getValue("dir");
		}
		else if (qName.equalsIgnoreCase("seq"))
		{
			mainRef.gestionPistes.seqDirectory = attrs.getValue("dir");
		}
		else if (qName.equalsIgnoreCase("snd"))
		{
			mainRef.gestionPistes.sndDirectory = attrs.getValue("dir");
		}
		else if (qName.equalsIgnoreCase("tk"))
		{
			mainRef.gestionPistes.tkDirectory = attrs.getValue("dir");
		}
		else if (qName.equalsIgnoreCase("tj"))
		{
			mainRef.gestionPistes.tjDirectory = attrs.getValue("dir");
		}
		else if (qName.equalsIgnoreCase("label"))
		{
			int number = new Integer(attrs.getValue("num"));
			String name = attrs.getValue("name");
			int color = new Integer(attrs.getValue("color"));
			if (number < Ut.INIT_TRACK_NB && !name.startsWith("Track"))
			{
				mainRef.gestionPistes.ts.labelAudio[number].setLabelName(name);
				mainRef.gestionPistes.ts.labelAudio[number].setColor(color);
				mainRef.gestionPistes.getTrack(number).setName(name);
			}
		}
		else if (qName.equalsIgnoreCase("last"))
		{
			lastOne = new HoloRecentFile(mainRef, attrs.getValue("dir"), attrs.getValue("name"), new Integer(attrs.getValue("type")));
			lastOne = lastOne.exists() ? lastOne : null;
		}
		else if (qName.equalsIgnoreCase("recent"))
		{
			mainRef.addRecent(new HoloRecentFile(mainRef, attrs.getValue("dir"), attrs.getValue("name"), new Integer(attrs.getValue("type"))));
		}
		else if (qName.equalsIgnoreCase("speaker"))
		{
			mainRef.gestionPistes.speakers.add(new HoloSpeaker(new Float(attrs.getValue("x")), new Float(attrs.getValue("y")), new Float(attrs.getValue("z")), new Float(attrs.getValue("d")), new Integer(attrs.getValue("num"))));
		}
		else if (qName.equalsIgnoreCase("trackselector"))
		{
			mainRef.gestionPistes.ts.setLocation(new Integer(attrs.getValue("x")), new Integer(attrs.getValue("y")));
			mainRef.gestionPistes.ts.setVisi(new Boolean(attrs.getValue("v")).booleanValue());
			mainRef.wbTrackSel = new Boolean(attrs.getValue("v")).booleanValue();
		}
		else if (qName.equalsIgnoreCase("transport"))
		{
			mainRef.transport.setLocation(new Integer(attrs.getValue("x")), new Integer(attrs.getValue("y")));
			mainRef.transport.setVisi(new Boolean(attrs.getValue("v")).booleanValue());
			mainRef.wbTrans = new Boolean(attrs.getValue("v")).booleanValue();
		}
		else if (qName.equalsIgnoreCase("soundpool"))
		{
			mainRef.soundPoolGui.setLocation(new Integer(attrs.getValue("x")), new Integer(attrs.getValue("y")));
			mainRef.soundPoolGui.setSize(new Integer(attrs.getValue("w")), new Integer(attrs.getValue("h")));
			mainRef.soundPoolGui.setVisi(new Boolean(attrs.getValue("v")).booleanValue());
			mainRef.soundPoolGui.setSplitDivPos(new Integer(attrs.getValue("split")));
			mainRef.wbSoundPool = new Boolean(attrs.getValue("v")).booleanValue();
		}
		else if (qName.equalsIgnoreCase("score"))
		{
			mainRef.score.setLocation(new Integer(attrs.getValue("x")), new Integer(attrs.getValue("y")));
			mainRef.score.setSize(new Integer(attrs.getValue("w")), new Integer(attrs.getValue("h")));
			mainRef.score.setVisi(new Boolean(attrs.getValue("v")).booleanValue());
			mainRef.wbScore = new Boolean(attrs.getValue("v")).booleanValue();
		}
		else if (qName.equalsIgnoreCase("room"))
		{
			mainRef.room.setLocation(new Integer(attrs.getValue("x")), new Integer(attrs.getValue("y")));
			mainRef.room.setSize(new Integer(attrs.getValue("w")), new Integer(attrs.getValue("h")));
			mainRef.room.setVisi(new Boolean(attrs.getValue("v")).booleanValue());
			mainRef.room.setProjView(attrs.getValue("viewProj"));
			mainRef.wbRoom = new Boolean(attrs.getValue("v")).booleanValue();
		}
		else if (qName.equalsIgnoreCase("room3d"))
		{
			mainRef.room3d.setLocation(new Integer(attrs.getValue("x")), new Integer(attrs.getValue("y")));
			mainRef.room3d.setSize(new Integer(attrs.getValue("w")), new Integer(attrs.getValue("h")));
			mainRef.room3d.setVisi(new Boolean(attrs.getValue("v")).booleanValue());
			mainRef.room3d.set3DView(attrs.getValue("view3D"));
			mainRef.wbRoom3D = new Boolean(attrs.getValue("v")).booleanValue();
		}
		else if (qName.equalsIgnoreCase("time"))
		{
			mainRef.timeEditor.setLocation(new Integer(attrs.getValue("x")), new Integer(attrs.getValue("y")));
			mainRef.timeEditor.setSize(new Integer(attrs.getValue("w")), new Integer(attrs.getValue("h")));
			mainRef.timeEditor.setVisi(new Boolean(attrs.getValue("v")).booleanValue());
			mainRef.wbTime = new Boolean(attrs.getValue("v")).booleanValue();
			mainRef.timeEditor.setCurves(attrs.getValue("curves"));
			mainRef.timeEditor.setTimeMode(new Integer(attrs.getValue("mode")));
		}
		else
		{
			Ut.print(" x Ce fichier n'a pas t cr avec Holo-Edit ou avec une version ultrieure de Holo-Edit !");
			Ut.print(" L'diteur ne comprend pas l'entre : " + qName);
		}
	}

	public void endElement(String uri, String localName, String qName)
	{
	}
}
