/*
 *  -----------------------------------------------------------------------------
 *  
 *  Holo-Edit, spatial sound trajectories editor, part of Holophon
 *  Copyright (C) 2006 GMEM
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 *  
 *  -----------------------------------------------------------------------------
 */
package holoedit.fileio;

import holoedit.Main;
import holoedit.data.HoloSDIFdata;
import holoedit.gui.ProgressBar;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Calendar;
import java.util.HashMap;
import java.util.TreeMap;
import java.util.Vector;
import javax.media.opengl.GL;

public class textFileReader implements Runnable
{
	private int error = NO_ERROR;
	public final static int USING_CACHE_ERROR = -1;
	public final static int NO_ERROR = 0;
	public final static int MONO_ERROR = 1;
	public final static int TYPE_ERROR = 2;
	public final static int LOAD_ERROR = 3;
	public final static int OCTET_ERROR = 4;
	public final static int INDEX_ERROR = 5;
	public final static int FILE_ERROR = 6;
	public final static int PATH_ERROR = 7;
	public final static int PATH2_ERROR = 8;
	public final static int SSDII_NORSRC_ERROR = 9;
	public final static int SSDII_RSRC_ERROR = 10;

	/** Pour l'affichage d'une barre de progression pendant l'ouverture du fichier*/
	private ProgressBar pb;
	private Thread runner;
	// SDIFdatas
	private HoloSDIFdata[] hSDIFdts;
	// state
	private boolean fine = false;
	private boolean done = false;
	// HoloSDIFdata attributs
	/** associated text file */
	public File textFile;
	private HashMap<Double, Vector<Vector<Double>>> hashMap = new HashMap<Double, Vector<Vector<Double>>>();
	
	public textFileReader(File f) {
		textFile = f;
		if (textFile.getName().length() > 32) {
			done = true;
			fine = false;
			error = PATH_ERROR;
			return;
		}
		pb = new ProgressBar("Importing text file...");
		pb.open();
		runner = new Thread(this);
		runner.setName("SF-Import");
		runner.setPriority(Thread.MAX_PRIORITY);
		runner.start();
	}
	
	public void run() {
		try {
			if (!done) {
				HoloSDIFdata data = new HoloSDIFdata(textFile);
				String[] st = {""};
				data.init(0, "TXT" , 0 , GL.GL_LINE_STRIP , st);
				BufferedReader reader = null;
				try {
					reader = new BufferedReader(new InputStreamReader(new FileInputStream(textFile)));
					String line;
					Vector<Double> vector;
					Vector<Vector<Double>> vector2;
					while((line = reader.readLine())!=null){
						String[] values = line.split(",");
						vector = new Vector<Double>();
						vector2 = new Vector<Vector<Double>>();
						for (int i=1; i<values.length; i++)
							vector.add(Double.parseDouble(values[i]));
						vector2.add(vector);
						hashMap.put(Double.parseDouble(values[0])*1000.0d, vector2);
					}
					createSDIFdata(data);
					reader.close();
				}catch (FileNotFoundException e) {
					e.printStackTrace();
				}catch (IOException e) {
					e.printStackTrace();
				}
			}
			pb.dispose();
			fine = true;
			done = true;
		} catch (Exception e) {
			error = LOAD_ERROR;
			pb.dispose();
			done = true;
			fine = false;
			e.printStackTrace();
			return;
		}
	}
	
	private void createSDIFdata(HoloSDIFdata data) {
		data.sdifTreeMap = new TreeMap<Double, Vector<Vector<Double>>>(hashMap);
		hSDIFdts = new HoloSDIFdata[1];
		int nbField = data.sdifTreeMap.get(data.sdifTreeMap.firstKey()).get(0).size();
		String[] fields = new String[nbField];
		for (int i=0; i<nbField; i++)
			fields[i] = "field"+i;
		double debut = data.sdifTreeMap.firstKey();
		double fin = data.sdifTreeMap.lastKey();
		data.setHoloDataStat();
		data.init(debut, "TXT", 0 , GL.GL_LINE_STRIP , fields);
		data.setEndTime(fin);
		data.setExtDataLength(fin - debut);
		data.setHoloDataStat();
		data.initialTreeMapSize = data.sdifTreeMap.size();
		data.setFine(true);
		hSDIFdts[0] = data;
	}
	
	public boolean isFine() {
		return fine;
	}
	public boolean isDone() {
		return done;
	}
	public int getError() {
		return error;
	}
	public HoloSDIFdata[] getHoloSDIFdatas() {
		return hSDIFdts;
	}
	@SuppressWarnings("deprecation")
	public void stop() {
		runner.stop();
	}
}
