/**
 *  -----------------------------------------------------------------------------
 *  
 *  Holo-Edit, spatial sound trajectories editor, part of Holophon
 *  Copyright (C) 2006 GMEM
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 *  
 *  -----------------------------------------------------------------------------
 */
package holoedit.gui;

import holoedit.Main;
import holoedit.data.HoloTrack;
import holoedit.data.HoloWaveForm;
import holoedit.fileio.WaveFormReader;
import holoedit.util.MacFileManager;
import holoedit.util.Ut;
import java.awt.BorderLayout;
import java.awt.Font;
import java.awt.MenuItem;
import java.awt.PopupMenu;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.dnd.DnDConstants;
import java.awt.dnd.DragGestureEvent;
import java.awt.dnd.DragGestureListener;
import java.awt.dnd.DragSource;
import java.awt.dnd.DropTarget;
import java.awt.dnd.DropTargetDragEvent;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.dnd.DropTargetEvent;
import java.awt.dnd.DropTargetListener;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;
import java.text.SimpleDateFormat;
import java.util.ConcurrentModificationException;
import java.util.Date;
import java.util.List;
import java.util.Vector;
import javax.swing.BorderFactory;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JSplitPane;
import javax.swing.JTextPane;
import javax.swing.SwingUtilities;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;

// FEATURE SOUNDPOOL PREECOUTE OU PATCH MAX SIMPLE
public class SoundPoolGUI extends FloatingWindow implements DropTargetListener, DragGestureListener
{
	private static Boolean supportsDnD;
	private Main mainRef;
	private DynamicTree soundTree;
	private DefaultMutableTreeNode selectedNode;
	private JTextPane info;
	private PopupMenu popup;
	private WaveFormRenderer waveView;
	private JSplitPane split, split2;
	private Font font = new Font("Default", Font.PLAIN, 9);
	private MenuItem importMI = new MenuItem("Import...");
	private MenuItem removeMI = new MenuItem("Remove waveform");
	private MenuItem updateMI = new MenuItem("Update");
	private MenuItem clearMI = new MenuItem("Clear");
	public static DataFlavor waveFlavor = new DataFlavor(HoloWaveForm.class, "HoloWaveForm");
	public boolean fine = false;
	public boolean done = false;
	public int error = WaveFormReader.NO_ERROR;
	public String errorFileName;
	public HoloWaveForm last;
	private Vector<File> unfound;
	private File soundFolder;
	private boolean choosingfolder;
	
	public SoundPoolGUI(Main owner)
	{
		super("Sound Pool", owner, owner.wsSoundPoolW, owner.wsSoundPoolH, owner.wlSoundPoolX, owner.wlSoundPoolY, owner.wbSoundPool);
		setResizable(true);
		mainRef = owner;
		soundTree = new DynamicTree("Imported Sounds");
		soundTree.addTreeSelectionListener(new WaveformTreeSelectionListener());
		popup = new PopupMenu();
		popup.add(importMI);
		popup.add(removeMI);
		popup.addSeparator();
		popup.add(updateMI);
		popup.add(clearMI);
		soundTree.add(popup);
		soundTree.addMouseListener(new MouseAdapter()
		{
			public void mousePressed(MouseEvent e)
			{
				if ((e.isControlDown() && Ut.MAC) || e.getButton() == MouseEvent.BUTTON3)
				{
					if (selectedNode == null)
					{
						removeMI.setEnabled(false);
						popup.show(soundTree, e.getX(), e.getY());
						return;
					}
					removeMI.setEnabled(selectedNode.isLeaf());
					popup.show(soundTree, e.getX(), e.getY());
					return;
				}
			}
		});
		removeMI.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				if (selectedNode.isLeaf())
				{
					int result = JOptionPane.showConfirmDialog(null,"This action will remove all instances of this waveform in the score\nand cannot be undone. Are you sure ?","Remove Waveform",JOptionPane.OK_CANCEL_OPTION);
					if(result == 0)
					{
						mainRef.modify();
						HoloWaveForm hwf = (HoloWaveForm) selectedNode.getUserObject();
						for(HoloTrack tk:mainRef.gestionPistes.tracks)
							tk.remove(hwf);
						for(HoloTrack tk:mainRef.gestionPistes.undoTracks)
							tk.remove(hwf);
						for(HoloTrack tk:mainRef.gestionPistes.redoTracks)
							tk.remove(hwf);
						mainRef.gestionPistes.undoTrack.remove(hwf);
						mainRef.gestionPistes.redoTrack.remove(hwf);
						mainRef.gestionPistes.soundPool.remove(hwf);
						soundTree.removeCurrentNode();
						selectedNode = null;
						mainRef.score.display();
					}
				}
			}
		});
		importMI.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				mainRef.gestionPistes.importSound();
			}
		});
		updateMI.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				soundTree.clear();
				updateTree();
			}
		});
		clearMI.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				soundTree.clear();
				mainRef.gestionPistes.soundPool.clear();
				updateTree();
			}
		});
		info = new JTextPane();
		info.setEditable(false);
		info.setText("Sound informations.");
		info.setFont(font);
		info.setOpaque(false);
		info.setEditable(false);
		info.setDragEnabled(true);
		info.setBorder(BorderFactory.createEmptyBorder(5, 10, 10, 10));
		waveView = new WaveFormRenderer(owner);
		waveView.setBorder(BorderFactory.createLoweredBevelBorder());
		split2 = new JSplitPane(JSplitPane.VERTICAL_SPLIT, soundTree, info);
		split2.setOneTouchExpandable(true);
		split2.setBorder(BorderFactory.createEmptyBorder());
		split = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, false, split2, waveView);
		split.setOneTouchExpandable(false);
		split.setDividerSize(0);
		split.setDividerLocation(250);
		split2.setDividerLocation(220);
		add(split, BorderLayout.CENTER);
		if (supportsDnD())
			makeDropTarget(soundTree.getTree());
		else System.out.println("Drag and drop is not supported with this JVM");
		unfound = new Vector<File>();
	}

	public void updateTree()
	{
		SwingUtilities.invokeLater(new Runnable()
		{
			public void run()
			{
				try
				{
					Vector<HoloWaveForm> sounds = mainRef.gestionPistes.soundPool.getSounds();
					soundTree.clear();
					for (HoloWaveForm h : sounds)
						soundTree.addObject(h);
					soundTree.repaint();
				}
				catch (NullPointerException e1)
				{
				}
				catch (ConcurrentModificationException e2)
				{
				}
			}
		});
	}

	public void clear()
	{
		soundTree.clear();
		mainRef.gestionPistes.soundPool.clear();
		unfound.clear();
	}

	public void repaint()
	{
		super.repaint();
	}

	private void makeDropTarget(final java.awt.Component c)
	{
		if (c.getParent() != null)
		{
			// Make drop target && drag source
			@SuppressWarnings("unused")
			final DropTarget dt = new DropTarget(c, this);
			final DragSource ds = new DragSource();
			ds.createDefaultDragGestureRecognizer(c, DnDConstants.ACTION_COPY_OR_MOVE, this);
		}
	}

	public void importSound(File f, boolean verboseError)
	{
			done = false;
			if(f.getName().length() <= 32)
			{
				Thread t = new Thread(new SoundFileImporter(f, verboseError));
				t.setPriority(Thread.MAX_PRIORITY);
				t.setName("SF-ImportDeamon");
				t.setDaemon(true);
				t.start();
			} else {
				if(verboseError)
					Ut.alert("Import Error", "Please rename the soundfile, for compatibility reasons with Max/MSP,\nit cannot be longer than 32 characters (including the extension).");
				error = WaveFormReader.PATH_ERROR;
				done = true;
				fine = false;
			}
	}
	class WaveformTreeSelectionListener implements TreeSelectionListener
	{
		public void valueChanged(TreeSelectionEvent e)
		{
			selectedNode = (DefaultMutableTreeNode) soundTree.getLastSelectedPathComponent();
			if (selectedNode == null)
				return;
			Object nodeInfo = selectedNode.getUserObject();
			if (selectedNode.isLeaf())
			{
				try
				{
					HoloWaveForm hwf = (HoloWaveForm) nodeInfo;
					info.setText("Name : " + hwf.getSoundFileName() + "\nPath : " + hwf.getSoundFile().getParent() + /*"\nChannel n : " + hwf.getChannel() +*/ "\nFileType : " + hwf.getFileType() + "\nSampleRate : " + hwf.getSampleRateMs() + "kHz" + "\nLength : "
							+ Ut.msToHMSMS(hwf.getFileLength()) + "\nLast modified : " + new SimpleDateFormat("dd/MM/yyyy 'at' HH:mm:ss").format(new Date(hwf.getSoundFile().lastModified())));
					waveView.setHoloWaveForm(hwf);
				}
				catch (ClassCastException cce)
				{
					info.setText("Sound informations.");
					waveView.setHoloWaveForm(null);
				}
			}
			else
			{
				info.setText("Sound informations.");
				waveView.setHoloWaveForm(null);
			}
		}
	}

	public void dragGestureRecognized(DragGestureEvent dge)
	{
		if(selectedNode != null)
			if(dge.getComponent().getClass().getName().endsWith("JTree") && selectedNode.isLeaf())
			{
				try
				{
					selectedNode = (DefaultMutableTreeNode) soundTree.getLastSelectedPathComponent();
					if (selectedNode == null)
						return;
					Object nodeInfo = selectedNode.getUserObject();
					if (selectedNode.isLeaf())
					{
						mainRef.score.droppedWaveForm = (HoloWaveForm) nodeInfo;
						mainRef.score.acceptDrag = true;
					}
				}
				catch (ClassCastException cce)
				{
					mainRef.score.droppedWaveForm = null;
					mainRef.score.acceptDrag = false;
				}
			}
	}

	public void dragEnter(DropTargetDragEvent dtde)
	{
		if (isDragOk(dtde))
			dtde.acceptDrag(java.awt.dnd.DnDConstants.ACTION_COPY);
		else dtde.rejectDrag();
	}

	public void dragOver(DropTargetDragEvent dtde)
	{
	}

	public void dropActionChanged(DropTargetDragEvent dtde)
	{
		if (isDragOk(dtde))
			dtde.acceptDrag(java.awt.dnd.DnDConstants.ACTION_COPY);
		else dtde.rejectDrag();
	}

	public void dragExit(DropTargetEvent dte)
	{
	}

	@SuppressWarnings("unchecked")
	public void drop(DropTargetDropEvent dtde)
	{
		try
		{
			Transferable tr = dtde.getTransferable();
			if (tr.isDataFlavorSupported(DataFlavor.javaFileListFlavor))
			{
				dtde.acceptDrop(java.awt.dnd.DnDConstants.ACTION_COPY);
				List fileList = (List) tr.getTransferData(DataFlavor.javaFileListFlavor);
				File[] filesTemp = new File[fileList.size()];
				fileList.toArray(filesTemp);
				final File[] files = filesTemp;
				filesDropped(files);
				dtde.getDropTargetContext().dropComplete(true);
			}
			else dtde.rejectDrop();
		}
		catch (java.io.IOException io)
		{
			System.out.println("FileDrop: IOException - abort:");
			io.printStackTrace();
			dtde.rejectDrop();
		}
		catch (java.awt.datatransfer.UnsupportedFlavorException ufe)
		{
			System.out.println("FileDrop: UnsupportedFlavorException - abort:");
			ufe.printStackTrace();
			dtde.rejectDrop();
		}
		finally
		{
		}
	}

	/** Determine if the dragged data is a file list. */
	private boolean isDragOk(final java.awt.dnd.DropTargetDragEvent evt)
	{
		boolean ok = false;
		DataFlavor[] flavors = evt.getCurrentDataFlavors();
		int i = 0;
		while (!ok && i < flavors.length)
		{
			if (flavors[i].equals(DataFlavor.javaFileListFlavor))
				ok = true;
			i++;
		}
		return ok;
	}

	public void filesDropped(File[] files)
	{
		Vector<File> filesToImport = new Vector<File>(1, 1);
		for (File f : files)
		{
			if (f.isDirectory())
			{
				File[] contentFile;
				if(Ut.MAC)
				{
					MacFileManager OSXFileManager = new MacFileManager(mainRef);
					contentFile = OSXFileManager.SoundFileFilter(f.listFiles());
				}
				else
				{
					contentFile = f.listFiles(mainRef.gestionPistes.sndFilter);
				}
				for (File cf : contentFile)
					filesToImport.add(cf);
			}
			else filesToImport.add(f);
		}
		for (File sndFile : filesToImport)
		{
			System.out.println("Importing " + sndFile.getAbsolutePath());
			importSound(sndFile, true);
		}
		mainRef.modify();
	}
	class SoundFileImporter implements Runnable
	{
		File f;
		boolean verbose = true;
		File cacheFile;

		public SoundFileImporter(File _f, boolean verb)
		{
			f = _f;
			verbose = verb;
			cacheFile = new File(Ut.dir(Ut.APP_PATH)+WaveFormReader.tmpdir+f.getName()+WaveFormReader.tmpser);
		}

		public void run()
		{
			if (f != null && ((f.exists() && f.isFile()) || (cacheFile.exists() && cacheFile.isFile())))
			{
				done = false;
				fine = false;
				if (!mainRef.gestionPistes.soundPool.contains(new HoloWaveForm(f)))
				{
					WaveFormReader wfr = new WaveFormReader(f,mainRef);
					try
					{
						while (!wfr.isDone())
						{
							Thread.sleep(250);
//							System.out.println("soundpool-import-waiting");
//							System.out.print('.');
						}
//						System.out.println();
					}
					catch (InterruptedException e)
					{
						e.printStackTrace();
					}
					done = true;
					if (wfr.isFine())
					{
						HoloWaveForm[] arr = wfr.getHoloWaveForms();
						for (HoloWaveForm hwf : arr)
							mainRef.gestionPistes.soundPool.add(hwf);
						updateTree();
						last = arr[arr.length-1];
						fine = true;
						if(wfr.getError() == -1)
							Ut.alert("File not found",f.getName()+" can't be found,\nusing it's cached waveform.");
					} else {
						error = wfr.getError();
						errorFileName = wfr.soundFile.getName();
						if(verbose)
							switch(error)
							{
							case WaveFormReader.NO_ERROR :
								break;
							case WaveFormReader.MONO_ERROR :
								Ut.alert("Error","Only mono file for instance,\nOnly the left channel has been read.");
								break;
							case WaveFormReader.TYPE_ERROR :
								Ut.alert("Import Error", errorFileName + " is not a supported file type.");
								break;
							case WaveFormReader.PATH_ERROR :
								Ut.alert("Import Error", "Please rename the soundfile, for compatibility reasons with Max/MSP,\nit cannot be longer than 32 characters (including the extension).");
								break;
							case WaveFormReader.PATH2_ERROR :
								Ut.alert("Import Error", "Please rename the soundfile, for compatibility reasons with Max/MSP,\nit cannot contain blank spaces.");
								break;
							case WaveFormReader.SSDII_NORSRC_ERROR :
								Ut.alert("Import Error", "This Sound Designer II file has no resource and cannot be read.");
								break;
							case WaveFormReader.SSDII_RSRC_ERROR :
								Ut.alert("Import Error", "There are errors in this Sound Designer II resource file.");
								break;
							default :
								Ut.alert("Import Error", "Problem while loading soundfile, aborted.");
								break;
							}
					}
					wfr.stop();
					wfr = null;
					System.runFinalization();
				} else {
					done = true;
					fine = true;
					last = new HoloWaveForm(f);
				}
			} else {
				done = true;
				fine = false;
				error = WaveFormReader.FILE_ERROR;
				unfound.add(f);
				if(!choosingfolder)
				{
					choosingfolder = true;
					Ut.alert("File not found", f.getName() + " can't be found,\neven it's cached waveform.\nLocalize ... ");
				} else {
					try{
						while(choosingfolder)
						{
//							System.out.println("soundpool-folder-localize-waiting");
							Thread.sleep(500);
						}
					} catch (InterruptedException e){}
				}
				if(soundFolder != null)
				{
					File[] sounds = soundFolder.listFiles();
					for (int k = unfound.size() - 1; k >= 0; k--)
					{
						File fil = unfound.get(k);
						for (int i = 0; i < sounds.length; i++)
						{
							if (sounds[i].getName().equalsIgnoreCase(fil.getName()))
							{
								mainRef.soundPool.importSound(sounds[i], true);
								unfound.remove(fil);
								mainRef.room.display();
								mainRef.modify();
							}
						}
					}		
				} else {
					JFileChooser chooser = new JFileChooser(f.getParentFile().exists() ? f.getParent() : mainRef.gestionPistes.holoDirectory);
					chooser.setAcceptAllFileFilterUsed(false);
					chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
					chooser.setDialogTitle("Choose a folder for missing sounds");
					chooser.showDialog(mainRef.soundPool, "Choose this folder for missing sounds");
					if (chooser.getSelectedFile() != null)
					{
						File tmp = chooser.getSelectedFile();
						if (tmp.exists() && tmp.isDirectory())
						{
							soundFolder = tmp;
							choosingfolder = false;
							File[] sounds = soundFolder.listFiles();
							for (int k = unfound.size() - 1; k >= 0; k--)
							{
								File fil = unfound.get(k);
								for (int i = 0; i < sounds.length; i++)
								{
									if (sounds[i].getName().equalsIgnoreCase(fil.getName()))
									{
										mainRef.soundPool.importSound(sounds[i], true);
										unfound.remove(fil);
										mainRef.room.display();
										mainRef.modify();
									}
								}
							}		
						}
					}
				}
				if(!unfound.isEmpty())
					Ut.alert("File not found", f.getName() + " can't still be found, aborted.");
			}
		}
	}

	public int getSplitDivPos()
	{
		return split2.getDividerLocation();
	}

	public void setSplitDivPos(int i)
	{
		split2.setDividerLocation(i);
	}

	public static boolean supportsDnD()
	{ // Static Boolean
		if (supportsDnD == null)
		{
			boolean support = false;
			try
			{
				Class.forName("java.awt.dnd.DnDConstants");
				support = true;
			} // end try
			catch (Exception e)
			{
				support = false;
			} // end catch
			supportsDnD = new Boolean(support);
		} // end if: first time through
		return supportsDnD.booleanValue();
	} // end supportsDnD

	public String toString()
	{
		return "\t<soundpool  split=\"" + getSplitDivPos() + "\"" + super.toString();
	}

	public void componentResized(ComponentEvent e)
	{
		super.componentResized(e);
		split.setDividerLocation(250);
	}
}
