/**
 *  -----------------------------------------------------------------------------
 *  
 *  Holo-Edit, spatial sound trajectories editor, part of Holophon
 *  Copyright (C) 2006 GMEM
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 *  
 *  -----------------------------------------------------------------------------
 */
package holoedit.util;

import holoedit.gui.HoloMenuBar;
import javax.swing.JOptionPane;


public class Ut
{
	public static void post(String s)
	{
		System.out.println(s);
	}
	public static int min(int a, int b)
	{
		return a <= b ? a : b;
	}

	public static int max(int a, int b)
	{
		return a >= b ? a : b;
	}

	public static int mean(int a, int b)
	{
		return ((a + b) / 2);
	}

	public static int mod(int value, int modulo)
	{
		while (value >= modulo)
			value -= modulo;
		return value;
	}

	public static int modabs(int value, int modulo)
	{
		while(value < 0)
			value += modulo;
		while (value >= modulo)
			value -= modulo;
		return value;
	}
	
	public static int clipL(int value, int lowlim)
	{
		return max(value, lowlim);
	}

	public static int clipU(int value, int uplim)
	{
		return min(value, uplim);
	}

	public static int clip(int value, int lowlim, int uplim)
	{
		return max(min(value, uplim), lowlim);
	}

	public static float min(float a, float b)
	{
		return a <= b ? a : b;
	}

	public static float max(float a, float b)
	{
		return a >= b ? a : b;
	}

	public static float mean(float a, float b)
	{
		return ((a + b) / 2);
	}

	public static float mod(float value, float modulo)
	{
		while (value >= modulo)
			value -= modulo;
		return value;
	}

	public static float clipL(float value, float lowlim)
	{
		return max(value, lowlim);
	}

	public static float clipU(float value, float uplim)
	{
		return min(value, uplim);
	}

	public static float clip(float value, float lowlim, float uplim)
	{
		return max(min(value, uplim), lowlim);
	}

	public static double min(double a, double b)
	{
		return a <= b ? a : b;
	}

	public static double max(double a, double b)
	{
		return a >= b ? a : b;
	}

	public static double mean(double a, double b)
	{
		return ((a + b) / 2);
	}

	public static double modSigned(double value, double modulo)
	{
		int sign = value >= 0 ? 1 : -1;
		value = Math.abs(value);
		while (value >= modulo)
			value -= modulo;
		return sign * value;
	}

	public static float modSigned(float value, float modulo)
	{
		int sign = value >= 0 ? 1 : -1;
		value = Math.abs(value);
		while (value >= modulo)
			value -= modulo;
		return sign * value;
	}
	
	public static int modSigned(int value, int modulo)
	{
		int sign = value >= 0 ? 1 : -1;
		value = Math.abs(value);
		while (value >= modulo)
			value -= modulo;
		return sign * value;
	}
	
	public static double mod(double value, double modulo)
	{
		while (value >= modulo)
			value -= modulo;
		return value;
	}

	public static double clipL(double value, double lowlim)
	{
		return max(value, lowlim);
	}

	public static double clipU(double value, double uplim)
	{
		return min(value, uplim);
	}

	public static double clip(double value, double lowlim, double uplim)
	{
		return max(min(value, uplim), lowlim);
	}

	public static boolean isPair(int i)
	{
		return mod(i,2) == 0;
	}
	
	public static double interpol(double f1, double f2, double step)
	{
		return (f1 + step * (f2 - f1));
	}

	public static float interpol(float f1, float f2, double step)
	{
		return (float) (f1 + step * (f2 - f1));
	}

	public static int interpol(int f1, int f2, double step)
	{
		return (int) (f1 + step * (f2 - f1));
	}

	public static boolean between(int v, int l, int u)
	{
		return v >= l && v <= u;
	}

	public static boolean between(float v, float l, float u)
	{
		return v >= l && v <= u;
	}

	public static boolean between(double v, double l, double u)
	{
		return v >= l && v <= u;
	}

	// date en centieme de secondes
	public static String intToDate(int dateNum)
	{
		String date;
		int heure = dateNum / 360000;
		int minute = (dateNum / 6000) % 60;
		int seconde = (dateNum / 100) % 60;
		int centiSeconde = dateNum % 100;
		date = "" + heure + ":";
		if (minute < 10)
			date = date + "0";
		date = date + minute + ":";
		if (seconde < 10)
			date = date + "0";
		date = date + seconde + ":";
		if (centiSeconde < 10)
			date = date + "0";
		date = date + centiSeconde;
		return (date);
	}

	public static String msToHMSMS(float ms)
	{
		float hf = ms / 3600000;
		int h = (int) hf;
		float mf = (hf - h) * 60;
		int m = (int) mf;
		float sf = (mf - m) * 60;
		int s = (int) sf;
		int rms = (int) ((sf - s) * 1000)	;
		if(rms != 0)
			rms += 1;
		String mss = ""+rms;
		while(mss.length() != 3)
			mss = "0"+mss;
		return h + ":" + m + ":" + s + "'" + mss + "\"";
	}

	public static int readInt4(byte[] array, int offset)
	{
		return (Ut.readUnsignedByte(array,offset) << 24) + (Ut.readUnsignedByte(array,offset+1) << 16) + (Ut.readUnsignedByte(array,offset+2) << 8) + Ut.readUnsignedByte(array,offset+3);
	}

	public static int readUnsignedByte(byte[] array, int offset)
	{
		return array[offset] >= 0 ? array[offset] : 256 + array[offset];
	}

	public static int readInt1(byte[] array, int offset)
	{
		return readUnsignedByte(array,offset);
	}

	public static int readInt2(byte[] array, int offset)
	{
		return (readUnsignedByte(array,offset) << 8) + readUnsignedByte(array,offset+1);
	}

	public static int readInt3(byte[] array, int offset)
	{
		return (readUnsignedByte(array,offset) << 16) + (readUnsignedByte(array,offset+1) << 8) + readUnsignedByte(array,offset+2);
	}

	public static String readString(byte[] array, int offset, int charNum)
	{
		String tmp = "";
		for(int i = 0 ; i < charNum ; i++)
			tmp = tmp + (char)array[offset+i];
		return tmp;
	}

	public static void alert(String title, String message)
	{
		JOptionPane.showMessageDialog(null, message, title, JOptionPane.ERROR_MESSAGE);
	}

	public static String APP_PATH;
	// boolean permettant de determiner l'environnement
	// cette distinction permet de compiler l'application sur n'importe quelle plate-forme
	// tout en permettant l'execution du code compile sur n'importe quelle plate-forme
	public static boolean MAC;
	// Les codes de caractere etant differents sous mac et pc, chaque fois qu'on
	// fait appel a certains d'entre eux
	// on vient les chercher ici.
	// Ce caracteres sert a la gestion des arborescences de fichiers
	public static String dirCar;
	public static String dir(String s)
	{
		if(s.endsWith(dirCar)) return s;
		return s + dirCar;
	}

	static public void print(String str)
	{
		System.out.println(str);
	}

	// le petit rond  / 
	public static char numCar;
	// son inverse pour garder la compatibilite en lecture de fichier (voir
	// TextRead > HP num (n, pos X, pos Y ...)
	public static char numCar2;
	public static String hv;
	public static double version = 4.1;
	//	public Application fApplication;
	static public HoloMenuBar barMenu;
	public static int drawPtsNb = 10;
	// Track Number
	public final static int INIT_TRACK_NB = 8;
	public final static int OLD_TRACK_NB = 17;
	public final static int MAX_TRACK_NB = 32;
	
	// GUI UPDATE MASK
	public final static int DIRTY_ALL = 1;
	public final static int DIRTY_ROOM = 2;
	public final static int DIRTY_ROOM3D = 4;
	public final static int DIRTY_SCORE = 8;
	public final static int DIRTY_TIME = 16;
	

	
	
}
