/**
 *  -----------------------------------------------------------------------------
 *  
 *  Holo-Edit, spatial sound trajectories editor, part of Holophon
 *  Copyright (C) 2006 GMEM
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 *  
 *  -----------------------------------------------------------------------------
 */
package holoedit;

import holoedit.data.HoloPoint;
import holoedit.data.HoloSpeaker;
import holoedit.data.HoloTrack;
import holoedit.data.HoloTraj;
import holoedit.data.HoloWaveForm;
import holoedit.data.WaveFormInstance;
import holoedit.gui.GestionPistes;
import holoedit.gui.SimpleSoundPool;
import holoedit.rt.Player;
import holoedit.util.Ut;
import java.awt.Color;
import java.io.File;
import java.util.Vector;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;
import com.cycling74.max.Atom;
import com.cycling74.max.Executable;
import com.cycling74.max.MaxClock;
import com.cycling74.max.MaxObject;
import com.cycling74.max.MaxSystem;

public class HoloPlayer extends MaxObject implements Executable,Player
{
	private static double version = 0.2;

	public GestionPistes gestionPistes;
	public SimpleSoundPool soundPool;
	protected static final int tick = 10;
	protected double tack;
	protected String vc,wc;
	double mytime, deltatack;
	protected double realcounterf;
	protected int realcounter;
	protected String session;
	protected MaxClock clock;
	protected boolean autostop = false;
	protected boolean playing = false;
	protected boolean paused = false;
	protected boolean loop = false;
	protected int counter;
	protected int endtime = 100000;
	protected int begtime = 0;
	protected int totaltime;
	protected boolean firstCue;
	protected boolean looping = false;
	protected boolean preloading;
	protected int loopNum = 0;
	protected HoloTrack currentTrack;
	protected Vector<HoloTraj> recTrajs = new Vector<HoloTraj>();
	protected int recBegDate = 0;
	protected boolean recording = false;
	protected boolean saved = true;
	

	public HoloPlayer()
	{
		initObj();
	}

	protected void initObj()
	{
		// On recupere le nom du system d'exploitation et on affecte les
		// variables en fonction
		Ut.MAC = System.getProperty("os.name").indexOf("Mac") >= 0;
		if (Ut.MAC)
		{
			Ut.dirCar = "/";
		} else {
			Ut.dirCar = "\\";
		}
		
		soundPool = new SimpleSoundPool(this);
		gestionPistes = new GestionPistes(this,false);
		clock = new MaxClock(this);
		declareAttribute("begtime","getBegin","setBegin");
		declareAttribute("endtime","getEnd","setEnd");
		declareAttribute("totaltime","getTotal","setTotal");
		declareAttribute("session","getSession","setSession");
		
		post("---------------------------------");
		post("");
		post("\t\tHolo-Edit session player v"+version);
		post("");
		post("---------------------------------");
	}
	
	// Demander confirmation a l'utilisateur avant de quitter si le fichier a ete modifie depuis sa derniere sauvegarde
	public boolean askForSave()
	{
		int result;
		boolean out = false;
		try
		{
			if (isSaved())
				out = true;
			else
			{
				result = JOptionPane.showConfirmDialog(null, "Do you want to save the changes", "Save ?", JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.WARNING_MESSAGE);
				if (result == 0)
				{
					gestionPistes.writeHoloFile();
					out = true;
				} else if (result == 1)
					out = true;
			}
		}
		catch (NullPointerException npe)
		{}
		return out;
	}

	public void init()
	{
		SwingUtilities.invokeLater(new Runnable()
		{
			public void run()
			{
				gestionPistes.initTracks2(false);
			}
		});
	}
	
	public void read()
	{
		SwingUtilities.invokeLater(new Runnable()
		{
			public void run()
			{
				gestionPistes.readHoloFile();
			}
		});
	}
	
	public void read(String fn)
	{
		session = fn;
		File f = new File(session);
		if(!f.exists())
			session = MaxSystem.locateFile(session);
		SwingUtilities.invokeLater(new Runnable()
		{
			public void run()
			{
				gestionPistes.readHoloFile(session);
			}
		});
	}
	
	public void write()
	{
		SwingUtilities.invokeLater(new Runnable()
		{
			public void run()
			{
				gestionPistes.writeHoloFileAs();
			}
		});
	}
	
	public void write(String fn)
	{
		session = fn;
		File f = new File(session);
		if(!f.exists())
		{
			session = MaxSystem.locateFile(session);
			System.out.println("correct : "+session);
		}
		SwingUtilities.invokeLater(new Runnable()
		{
			public void run()
			{
				gestionPistes.writeHoloFile(session);
			}
		});
	}
	
	public void writeagain()
	{
		SwingUtilities.invokeLater(new Runnable()
		{
			public void run()
			{
				gestionPistes.writeHoloFile();
			}
		});
	}
	
	public void out(String s)
	{
		outlet(0,Atom.newAtom(s));
	}
	
	public void done()
	{
		outlet(0,Atom.newAtom("done"));
	}
	
	public void position()
	{
		counter = begtime;
		out("time "+counter);
		for(int i = 0, last = gestionPistes.getNbTracks() ; i < last ; i++)
		{
			currentTrack = gestionPistes.getTrack(i);
			if(currentTrack.isVisible())
			{
				String vc1 = currentTrack.getLastValueAt(counter);
				if(!vc1.equalsIgnoreCase("none"))
					out("tk "+currentTrack.getNumber()+" pos "+vc1);
			}
		}
	}
	
	public void sendVisible()
	{
		String tkVisi = "visible";
		for(HoloTrack t:gestionPistes.tracks)
			tkVisi = tkVisi + " " + t.getNumber() + " " + (t.isVisible() ? 1 : 0);
		out(tkVisi);
	}

	public void sendPaths()
	{
		Vector<String> paths = new Vector<String>();
		for(HoloWaveForm hwf:gestionPistes.soundPool.getSounds())
		{	
			String p = hwf.getPath();
			if(!paths.contains(p))
				paths.add(p);
		}
		out("paths "+paths.size());
		for(int i = 0 ; i < paths.size() ; i++)
			out("path "+i+" "+paths.get(i));
	}
	
	public void play()
	{
		if(!playing)
		{
			// PRELOAD
			preload();
			try {
				Thread.sleep(500);
			} catch (InterruptedException e) {}
			position();
			for (HoloTrack tk : gestionPistes.tracks)
			{
				if (tk.waves.isEmpty())
					totaltime = Ut.max(totaltime, tk.getLastDate());
				else
					totaltime = Ut.max(totaltime, Ut.max(tk.getLastDate(), tk.waves.lastElement().getLastDate()));
			}
			autostop = false;
			playing = true;
			firstCue = true;
			loopNum = 0;
			counter = begtime;
			realcounterf = realcounter = begtime;
			looping = !firstCue;
			tack = System.currentTimeMillis()/10. - tick;
			clock.delay(0);
		}
	}
	
	public void pause()
	{
		if(paused)
		{
			paused = false;
			tack = System.currentTimeMillis()/10. - tick;
			clock.delay(0);
		} else {
			paused = true;
			clock.unset();
		}
	}
	
	public void resume()
	{
		if(paused)
		{
			paused = false;
			tack = System.currentTimeMillis()/10.;
			clock.delay(tick);
		}
	}
	
	public void stop()
	{
		clock.unset();
		if(recording)
			record(0);
		recording = false;
		playing = false;
		paused = false;
	}
	
	protected void setLoop(boolean loop)
	{
		this.loop = loop;
	}
	
	protected boolean isLoop()
	{
		return loop;
	}
	
	public void preload()
	{
		preloading = true;
		sendPaths();
		sendVisible();
		for(HoloTrack tk:gestionPistes.tracks)
			for(int i = 1 ; i <= tk.waves.size() ; i++)
				tk.waves.get(i-1).preload2(tk,i+100,begtime,begtime,endtime);
		preloading = false;
	}
	
	public void update()
	{
		String tmp = "tks "+gestionPistes.getNbTracks();
		for(HoloTrack tk:gestionPistes.tracks)
			tmp += " " + tk.getNumber();
		out(tmp);
		for(HoloTrack tk:gestionPistes.tracks)
		{
			Color c = tk.getColor();
			String na = tk.getName();
			if(na.indexOf(' ') != -1)
				na = "\""+na+"\"";
			out("tkup "+tk.getNumber()+" "+na+" "+c.getRed()+" "+c.getGreen()+" "+c.getBlue());
		}
		position();
		preload();
	}
	
	public void execute()
	{
		while(preloading)
		{
			System.out.println("preloading...");
			try {
				Thread.sleep(10);
			} catch (InterruptedException e) {}
		}
		if(!paused && playing)
		{
			while (realcounter >= counter)
			{
				if(loop && counter == endtime+1)
				{
					out("looping");
					counter = begtime;
					looping = true;
					loopNum++;
				} else if(counter > totaltime && endtime <= totaltime && !recording) {
					out("stop");
					playing = false;
					paused = false;
					autostop = true;
				}
					
				out("time "+counter);
				String c = "cues";
				int count = 1;
				for(int i = 0, last = gestionPistes.getNbTracks() ; i < last ; i++)
				{
					currentTrack = gestionPistes.getTrack(i);
					if(currentTrack.isVisible())
					{
						if((recording && !currentTrack.isRecEnable()) || !recording)
						{
							vc = currentTrack.getValueAt(counter,autostop,looping);
							//vc = "0. 0. 0. 0.";
							if(!vc.equalsIgnoreCase("none"))
								out("tk "+currentTrack.getNumber()+" pos "+vc);
						}
						for(WaveFormInstance w : currentTrack.waves)
						{	
							wc = w.play(currentTrack, counter, firstCue, loop, looping);
							if(!wc.equalsIgnoreCase("none"))
								c = c + " " + wc;
						}
					}
				}
				if(!c.equalsIgnoreCase("cues") && playing)
					out(c);
				firstCue = false;
				looping = false;
				counter++;
			}
			if(!autostop)
			{ 
				mytime = System.currentTimeMillis()/10.;
				deltatack = mytime - tack;
				realcounterf += deltatack;
				realcounter = (int)realcounterf;
				clock.delay(tick);
				//out("debug "+counter+" "+realcounter+" "+realcounterf+" "+mytime);
				tack = mytime;
				
				
				//out("debug "+System.currentTimeMillis());
			}
			
		}
	}
	
	public void notifyDeleted()
	{
		clock.unset();
	}
	
	public void setBegin(int i)
	{
		begtime = i >= 0 ? i : 0;
		out("begin "+begtime);
	}
	
	public int getBegin()
	{
		return begtime;
	}
	
	public void setEnd(int i)
	{
		endtime = i >= 0 ? i : 0;
		out("end "+endtime);
	}
	
	public int getEnd()
	{
		return endtime;
	}
	
	public void setTotal(int i)
	{
		totaltime = i >= 0 ? i : 0;
		out("total "+totaltime);
	}
	
	public int getTotal()
	{
		return totaltime;
	}
	
	public void setSession(String filename)
	{
		session = filename;
		read(session);
	}
	
	public int setSessionName(String filename)
	{
		session = filename;
		return 0;
	}
	
	public String getSession()
	{
		return session;
	}
	
	public void record(int r)
	{
		setSaved(false);
		if(r == 0)
		{
			// stop recording
			for(int i = 0 ; i < recTrajs.size() ; i++)
			{
				HoloTraj ht = recTrajs.get(i);
				ht.remove(0);
			}
			for(HoloTrack tk:gestionPistes.tracks)
				tk.update();
			recording = false;
			out("record 0");
		} else {
			// start recording
			saved = false;
			recTrajs = new Vector<HoloTraj>();
			recBegDate = isPlaying() ? counter : begtime;
			for(int i = 0 ; i < gestionPistes.getNbTracks() ; i++)
			{
				HoloTraj ht = new HoloTraj();
				currentTrack = gestionPistes.getTrack(i);
				ht.add(new HoloPoint(0,0,0,recBegDate,true));
				recTrajs.add(ht);
				if(currentTrack.isVisible() && !currentTrack.isLocked() && currentTrack.isRecEnable())
				{
					currentTrack.addTraj(ht);
					currentTrack.update();
				}
			}
			recording = true;
			out("record 1");
		}
	}
	
	public boolean isRecording()
	{
		return recording;
	}
	
	public boolean isPlaying()
	{
		return playing;
	}
	
	protected void treatRecord(String[] msg)
	{
		int loopDur = endtime - begtime;
		int tkNum = Integer.parseInt(msg[0]);
		float x = Float.parseFloat(msg[1]);
		float y = Float.parseFloat(msg[2]);
		float z = Float.parseFloat(msg[3]);
		try
		{
			int date = Integer.parseInt(msg[4]);
			boolean found = false;
			for(int p = 0 ; p < gestionPistes.getNbTracks() && !found ; p++)
			{
				HoloTrack tk = gestionPistes.getTrack(p);
				if(tk.getNumber()==tkNum && tk.isRecEnable())
				{
					found = true;
					recTrajs.get(p).add(new HoloPoint(x,y,z,date+loopNum*loopDur,false));
				}
			}
		} catch (ArrayIndexOutOfBoundsException e) {}
	}
	
	public void setSaved(boolean b)
	{
		saved = b;
	}
	
	public boolean isSaved()
	{
		return saved;
	}
	
	public void anything(String msg, Atom[] args)
	{
		if(msg.equalsIgnoreCase("rec") && recording)
		{
			treatRecord(Atom.toString(args));
		} else if(msg.equalsIgnoreCase("recenable")) {
			if(!args[0].toString().equalsIgnoreCase("#1"))
			{
				int tkNum = args[0].toInt();
				boolean tkRecOn = args[1].toInt() > 0;
				boolean found = false;
				for(int p = 0 ; p < gestionPistes.getNbTracks() && !found ; p++)
				{
					HoloTrack tk = gestionPistes.getTrack(p);
					if(tk.getNumber()==tkNum)
					{
						found = true;
						tk.setRecEnable(tkRecOn);
					}
				}
			}
		} else if(msg.equalsIgnoreCase("tk")) {
			if(!args[0].toString().equalsIgnoreCase("#1"))
			{
				int i = args[0].toInt();
				boolean o = args[1].toInt() > 0;
				boolean found = false;
				for(int p = 0 ; p < gestionPistes.getNbTracks() && !found ; p++)
				{
					HoloTrack tk = gestionPistes.getTrack(p);
					if(tk.getNumber()==i)
					{
						found = true;
						tk.setVisible(o);
					}
				}
			}
		} else if(msg.equalsIgnoreCase("queryspeakers")) {
			sendSpeakers();
		}
	}

	public void sendSpeakers()
	{
		HoloSpeaker sp;
		for(int i = 0 ; i < gestionPistes.speakers.size() ; i++)
		{
			sp = gestionPistes.speakers.get(i);
			out("spk "+(i+1)+" "+sp.X+" "+sp.Y+" "+sp.Z);
		}
	}
	
	public SimpleSoundPool getSoundPool()
	{
		return soundPool;
	}
}